// B4R Library to draw 20 x 16 pixel bitmaps on a LCD1602.
// Changelog
// 20190707: NEW
#pragma once
#include "B4RDefines.h"
#include "rLiquidCrystal.h"
#include "LCDBitmap.h"
namespace B4R {
	//~Version: 1.0
	//~shortname: LcdBitmap
	class B4RLCDBitmap {
		private:
			LCDBitmap* bn;
			uint8_t be[sizeof(LCDBitmap)];
		public:

		/**
		*Assign LCDBitmap to lcd, set bitmap display location to character position (x,y)
		*Initializes the object with the LCD display from library rLiquidCrystal (must be initialized prior to initializing LCDBitmap)
		*Example:<code>
		*lcd.Initialize(8,255,9, Array As Byte(4,5,6,7)) 
		*lcd.Begin(16,2)
		*bitmap.Initialize(lcd, 0, 0)</code>
		*Note: (lcd, x, y ) - lcd is from rLiquidCrystal, (x,y) is character position of the bitmap, 
		*0,0 = top/left, 12,0 = far right on a 16x2 LCD. The 20x16 pixel display that LCDBitmap creates is 4 characters wide by 2 characters high.
		*
		*/		
		void Initialize(B4RLiquidCrystal* lcd, byte bitmap_x, byte bitmap_y);
		/**
		*
		*Initialize the LCD bitmap
		*
		*/
		void begin();
		/**
		*
		*Clear the bitmap display (automatically updates bitmap display) doesn't clear text 
		*
		*/
		void clear();
		/**
		*
		*Invert the bitmap, automatically updates the bitmap display 
		*
		*/		
		void inverse();
		/**
		*
		*Update bitmap display 
		*
		*/				
		void update();
		/**
		*
		*Clear just the text on the display (leaves bitmap alone) 
		*
		*/				
		void clear_text();
		/**
		*
		*Move cursor the home position (0,0). Does a lcd.SetCursor(0,0), just added for convenience  
		*
		*/				
		void home();
		/**
		*
		*Move the LCD bitmap position to this character position 
		*
		*/			
		void move(byte x, byte y);	
		/**
		*
		*Add pixel at (x,y), color is either True or False, update is either True or False 
		*
		*/		
		void pixel(byte x, byte y, bool color, bool update=false);
		/**
		*
		*Draw the line from (x1,y1) to (x2,y2), color and update as in pixel 
		*
		*/				
		void line(byte x1, byte y1, byte x2, byte y2, bool color, bool update=false);
		/**
		*
		*Draw a rectangle from (x1,y1) to (x2,y2), color and update as in pixel  
		*
		*/				
		void rect(byte x1, byte y1, byte x2, byte y2, bool color, bool update=false);
		/**
		*
		*Draw a filled rectangle from (x1,y1) to (x2,y2), color and update as in pixel
		*
		*/
		void rectFill(byte x1, byte y1, byte x2, byte y2, bool color, bool update=false);
		/**
		*
		*Draw bar graph, bars is no of bars (1,2,4,5,10,20), graph is array containing height values, color and update as in pixel 
		*
		*/		
		void barGraph(byte bars, ArrayByte* graph, bool color, bool update=false);	
		/**
		*
		*Can be maximum 20 pixels wide (or 4 LCD chars wide)
		*
		*/				
        byte BitmapW = 20;
		/**
		*
		*Can be maximum 16 pixels high (or 2 LCD chars high)
		*	
        */		
		byte BitmapH = 16;	
		/**
		*
		*Pixels ON
		*	
        */		
		bool COLOR_ON = true;			
		/**
		*
		*Pixels OFF
		*	
        */		
		bool COLOR_OFF = false;		
		/**
		*
		*Update ON
		*	
        */		
		bool UPDATE_ON = true;	
		/**
		*
		*Update OFF
		*	
        */		
		bool UPDATE_OFF = false;			
		
};

}